struct position quantis (struct position x, struct SS SS) 
{     
	/*
	 Quantisatition of a position
	 Only values like x+k*q (k integer) are admissible 
	 */ 
	int d;
	double qd;
	struct position quantx;

	quantx = x;     

	for (d = 0; d < x.size; d++)
	{
		qd = SS.q.q[d];	

		if (qd > zero)	// Note that qd can't be < 0
		{     	      
			quantx.x[d] = qd * floor (0.5 + x.x[d] / qd);	    
		}
	}
	return quantx;    
}
// ===========================================================
struct quantum quantAdapt(struct swarm SW, struct problem pb, int quant)
{
	struct quantum qAdapt;
	int d;
	double xMin, xMax;
	int s;

	qAdapt.size=pb.SS.q.size;

	switch(quant)
	{
		default: // Do nothing
			return pb.SS.q;

		case 1: // Variable. Should decrease if convergence
			qAdapt=pb.SS.q;
		for (d = 0; d < pb.SS.D; d++)  // On each dimension
			// find the min and max position 
		{
			xMin= SW.X[0].x[d];
			xMax= SW.X[0].x[d];		
			for(s=1;s < SW.S; s++)
			{
				if(SW.X[s].x[d]<xMin) xMin=SW.X[s].x[d];
				if(SW.X[s].x[d]>xMax) xMax=SW.X[s].x[d];
			}

			qAdapt.q[d]=0.5*(xMax-xMin)/(SW.S+1);
		}
		break;

		case 2: // Decreasing over time
			for (d = 0; d < pb.SS.D; d++) 
		{
			qAdapt.q[d]=(pb.SS.max[d]-pb.SS.min[d])/pow(2,(double)iter/SW.S);
		}
		break;
	}
	for (d = 0; d < pb.SS.D; d++) 
	{
		if(pb.SS.q.q[d]>zero) // There is already a quantum
			// The new one must be k (integer) times the previous one
		{
			qAdapt.q[d]=pb.SS.q.q[d]+(int)(qAdapt.q[d]/pb.SS.q.q[d]);
		}
	}
	return qAdapt;
}

// ===========================================================
struct position valueAccept(struct position x, int valueNb)
{
	// valueList[] is a global variable
	// Move the position to the nearest acceptable value, 
	// for each dimension independently
	int d;
	int i;

	struct position xv;
	xv=x;

	for(d=0;d<x.size;d++)
	{
		if(x.x[d]<= valueList[0])
		{
			xv.x[d]=valueList[0];
			continue;
		}

		if(x.x[d]>= valueList[valueNb-1])
		{
			xv.x[d]=valueList[valueNb-1];
			continue;
		}


		for(i=1;i<valueNb;i++)
		{
			if(x.x[d]>=valueList[i-1] && x.x[d]<=valueList[i] )
			{	
				if(x.x[d]-valueList[i-1]<valueList[i]-x.x[d])
					xv.x[d]=valueList[i-1];
				else
					xv.x[d]=valueList[i];			
				break;
			}
		}
	}
	return xv;
}
// ======================================================
struct position discrete(struct position pos0, struct problem pb, int quant,
    struct quantum qAdapt)
{
	struct position pos;
	struct quantum qTemp;

	if(pb.SS.valueNb>0) // The search space is a list of values
	{
		return valueAccept(pos0,pb.SS.valueNb);
}
	if(quant>0) 
	{	
	// Quantisation
	qTemp=pb.SS.q; // Temporarily save the quantisation
	pb.SS.q=qAdapt;
	pos = quantis (pos0, pb.SS);		
	pb.SS.q=qTemp;
	return pos;
	}
	else return pos0;
}
//========================================================
struct xv confinement(struct xv xv0,  
    struct problem pb,struct param param,int R)
{
// Confinement and evaluation
// Note: the two are together, for depending on the clamping option
// 				there may be no evaluation at all
	int d;
	int eval;
	struct fitness ff; // Just for constraints
	int i;
	int outside;
	struct xv xv;

	xv=xv0;

	switch (param.clamping)
	{			
		case 0:	// No clamping AND no evaluation if outside
			outside = 0;

		for (d = 0; d < pb.SS.D; d++)
		{			
			if(xv.x.x[d] < pb.SS.min[d] || xv.x.x[d] > pb.SS.max[d])
				outside++;		
		}

		if (outside == 0) eval=1;	// If inside, the position is evaluated
		else eval=0;

		break;

		case 1:	// Set to the bounds, and v to zero
			outside=0;
		eval=1;
		for (d = 0; d < pb.SS.D; d++)
		{	
			if (xv.x.x[d] < pb.SS.minS[d])
			{	
				xv.x.x[d] = pb.SS.minS[d];
				xv.v.v[d] = 0;
			}

			if (xv.x.x[d] > pb.SS.maxS[d])
			{			
				xv.x.x[d] = pb.SS.maxS[d];
				xv.v.v[d] = 0;			
			}			
		}
		break;	
		
		case 2: // No clamping, no evaluation, velocity modified
		outside = 0;

		for (d = 0; d < pb.SS.D; d++)
		{			
			if(xv.x.x[d] < pb.SS.min[d] || xv.x.x[d] > pb.SS.max[d])
				outside++;
				xv.v.v[d]=	0.99*xv.v.v[d];	
		}

		if (outside == 0) eval=1;	// If inside, the position is evaluated
		else eval=0;
		
		break;		
	}

	xv.x=quantis(xv.x,pb.SS);

	if(pb.constraint==0)  goto evaluation;

	ff=constraint(xv.x,pb.function,pb.epsConstr);

		for(i=1;i<ff.size;i++)
	{
		if(ff.f[i]<=0) continue;

		// If a constraint is not respected
		// the velocity is decreased
		// and the position moved accordingly
		// in th HOPE that the new position will be OK
		// (but we don't check it)
		for(d=0;d<pb.SS.D;d++) 
		{
			xv.v.v[d]=xv.v.v[d]/2;
			xv.x.x[d]=xv.x.x[d]-xv.v.v[d];
		}
			xv.x=quantis(xv.x,pb.SS);	
	}

		evaluation:	// Evaluation

			if(eval==1)
	{
		xv.x.f =
			perf (xv.x, pb,R, param.aroundNb, param.S,param.normalise);						
	}
	else xv.x.f=xv0.x.f;
	
		return xv;
}
//==========================================================
struct fitness constraint(struct position x, int functCode, double epsConstr)
{
	// ff[0] is defined in perf()
	// Variables specific to Coil compressing spring
	static double	Fmax=1000.0;
	static double	Fp=300;
	double Cf;
	double K;
	double sp;
	double lf;

	static double	S=189000.0;
	static double	lmax=14.0;
	static double	spm=6.0;
	static double	sw=1.25;
	static double	G=11500000;
	struct fitness ff={0};
	ff.size=1; // Default value

	switch(functCode)
	{

		case 1007:
			ff.size=4;
		case 7:
			ff.f[1]=0.0193*x.x[2]-x.x[0];
			ff.f[2]=0.00954*x.x[2]-x.x[1];
			ff.f[3]=750*1728-pi*x.x[2]*x.x[2]*(x.x[3]+(4.0/3)*x.x[2]); 
			break;

		case 1008:
			ff.size=5;
		case 8:
			Cf=1+0.75*x.x[2]/(x.x[1]-x.x[2]) + 0.615*x.x[2]/x.x[1];
			K=0.125*G*pow(x.x[2],4)/(x.x[0]*x.x[1]*x.x[1]*x.x[1]);
			sp=Fp/K;
			lf=Fmax/K + 1.05*(x.x[0]+2)*x.x[2];

			ff.f[1]=8*Cf*Fmax*x.x[1]/(pi*x.x[2]*x.x[2]*x.x[2]) -S;
			ff.f[2]=lf-lmax;
			ff.f[3]=sp-spm;			
			ff.f[4]=sw- (Fmax-Fp)/K;
			break;

		case 1015:
			ff.size=4 ;
		case 15:
			ff.f[1]=fabs(x.x[0]*x.x[0]+x.x[1]*x.x[1]+x.x[2]*x.x[2]
			    +x.x[3]*x.x[3]+x.x[4]*x.x[4]-10)-epsConstr; // Constraint h1<=eps
			ff.f[2]=fabs(x.x[1]*x.x[2]-5*x.x[3]*x.x[4])-epsConstr; // Constraint h2<=eps;
			ff.f[3]=fabs(pow(x.x[0],3)+pow(x.x[1],3)+1)-epsConstr; // Constraint h3<=eps
			break;

	}

	return ff;
	}