/*
 Balanced PSO
 Research version
 Can "simulate" Standard PSO 2007, and a lot of other PSO variants
 The purpose of this program is to define a lot of options, and to "play"
 with them.

 Contact for remarks, suggestions etc.:
 Maurice.Clerc@WriteMe.com
 PS. Don't tell me the code is not optimised, I know  ;-)
But if you find a bug, or want to suggest another option, don't hesitate!


 Last update. The date is also the name of the version
 2010-03-17 Removed an useless constraint in Compression spring
 						(pointed out by R. Fromato). Does not change the results 
 2010-03-09 Variable swarm size
 2010-03-08 Initialisation on the boundaries. Not convincing
 2010-03-07 Test of a "parameter-free" PSO (pf-PSO by Ramana Murthy). Very bad
 2010-02-27 Hybrid strategies + stagnation detections
 2010-02-26 Orthogonal Walk (OW). Sometimes extremely good ... and sometimes
 						very bad
 2010-02-18 Option: quasi non-sensitivity to rotation (by using hyperspheres)
 2010-02-14 Orthogonal Learning (OL), and Factor analysis (FA) 
 						Not as good as sometimes claimed (Zhan) 
 						but on the whole a slight improvement
 						Note: FA is almost useless, for in practice the
 						combination it finds has already been used in the OL. Actually,
 						the best way is often to _not_ use it
 						
 2010-02-11 Variable ring topology. Very slightly more robust than SPSO
 						Results are often slightly worse but also sometimes better. 					
 2010-02-10 Evaluate how each particle can "trust" its informants (confidence)
 						and the topology can be modified according to this measure
 						(not convincing)
 2010-02-04 Seriously modified the "opposite" option. Still not convincing
 2010-01-05 Mutation options (for coefficients w, c1, and c2)
 2010-01-04 Inheritance option. Each particle can "learn" its coefficients
 						w, c1 and c2 from better ones
 2010-01-01 Largely rewritten. More modular. 
 						WARNING: the way Local Exploitation Areas are used has been modified.
 						So the "suggested" parameter values may be different, compared
 						to previous versions 
            
 2009-12-29 fixed a small bug in the Schwefel function (code 12)
 2009-10-05 option: Normalised search space [0,1]^D. Should change nothing 
						as long as the real search space is a hypercube, but also
						as long as each dimension is considered independently 
 2009-09-18 Reinitialisation of the worst particles (Angeline 1998)
 2009-09-09 Added a Schwefel function (code 12)
 2009-09-09 Fixing a bug in the Hammersley initialisation
 2009-05-24 Convex "small" polygon, smallest perimeter, 4 edges
 2009-04-05 Cutting stock problem
 2009-01-22 Cellular phone problem
 2009-01-18 Gear train problem
 2009-01-12 FEmaxSize parameter. Now, one can have far more quickly the curves
 "success probability" vs "number of fitness evaluations" and
 "mean best" vs "number of fitness evaluations". 
 2009-01-09 Pressure vessel, compression spring problems
 2009-01-05 CEC 2005 F2, F7, F9 (not rotated)
 2008-12-27 Feasible space (FS), included in the search space (SS). Confinement inside the 
 search space, and no evaluation for positions in SS but not in FS
 2008-12-22 Two strategies (only different w, for the moment)
 2008-12-14 Biased random initialisation
 2008-12-01 Min potential initialisation
 2008-11-24 Hammersley initialisation
 2008-10-31 Gaussian local search
 2008-10-13 Three CEC 2005 functions (F1, F6, F9)

TO DO  (also, search for "TO DO" in the code)
- option (+dx, -dx) one dimension at a time. If no improvement, dx=dx/2, as long as dx>dx_min
  If still no improvement, re-initialise in a "no-mans-land" (see SunnySpell option of Tribes)
  Peut-être mettre un peu de hasard sur dx 
- automatic set of parameters, depending on the dimension of the problem
	For example no "Orthogonal learning + FA" in low dimension

- a few things to completely test "Krigin" ... or to remove it!
- option: mean best position instead of gbest (Clerc 1999, Sun 2004)

 ----------------------------------- Basic parameters
 S := swarm size
 K := maximum number of particles _informed_ by a given one
 w := first cognitive/confidence coefficient
 c := second cognitive/confidence coefficient

Suggested values are either hard coded or automatically computed.

 ----------------------------------- Equations of the "normal" move
 For each particle and each dimension
 Equation 1:	v(t+1) = w*v(t) + R(c1)*(p(t)-x(t)) + R(c2)*(g(t)-x(t))
 Equation 2:	x(t+1) = x(t) + v(t+1)
 where
 v(t) := velocity at time t
 x(t) := position at time t
 p(t) := best previous position of the particle
 g(t) := best position amongst the best previous positions
 of the informants of the particle
 R(c) := a number coming from a random distribution, which depends on c
 In SPSO 2007, the distribution is uniform on [0,c]

 Note 1:
 When the particle has no informant better than itself,
 it implies p(t) = g(t)

 Equation 1 is then modified:
 v(t+1) = w*v(t) + R(c)*(p(t)-x(t))  
and NOT   v(t+1) = w*v(t) + 2*R(c)*(p(t)-x(t))
 
 ----------------------------------- Options
There are a lot of options, paramBPSO.c for details.

 Each option has a "code", which points out the difference with SPSO 2007.
This code is displayed at the end of the run. For example: 
 PSO P2 V3
means
 "SPSO 2007, but with option 2 (improved Hammersley) for the initialisation of the positions,
 and option 3 (set to zero) for the initialisation of the velocities

 ----------------------------------- Miscellaneous
- the search space can be any real hyperparallelepid, but the values can be discretised
	(granularity)
- the search space can also be a list of values (reads from a file)
- WARNING Some combinations of options are "stupid". Sometimes I check the possible
	inconsistencies, sometimes not. Be careful.


 */

#include "main.h" // Structures, subprogramme list, global variables
// =================================================
int main () 
{ 
	struct position bestBest;
	double convRateMean;
	int d;					// Current dimension
	int dim;
	double eps;
	double error;				// Current error
	double errorMean;			// Average error
	double errorMin=infinity;	// Best result over all runs
	double errorMeanBest[RMax]; 
	double evalMean;			// Mean number of evaluations
	int functionCode;
	int j;
	//int initLink;
	int n;
	int nFailure;				// Number of unsuccessful runs
	double logProgressMean=0;
	struct param param={0};
	struct problem pb={0};
	int  runMax; 
	struct result result;
	int s,s1; 
	int SPSO; 				// For using different sets of parameters
	double successRate;
	double variance;
	int wSize;
	double zz,zz1;


	f_run = fopen ("f_run.txt", "w");  
	f_synth = fopen ("f_synth.txt", "w"); 
	f_expl = fopen ("f_expl.txt", "w"); // Just for info (exploitation rate)
	fLandscape=fopen("fLandscape.txt","r"); // When data are on a file
	f_init_save=fopen("f_init_save.txt","w");
	fCurve=fopen("fCurve.txt","w");
	f_vario = fopen ("f_vario.txt", "w");  
	f_coeff= fopen("f_coeff.txt","w");
	f_confid=fopen("f_confid.txt","w"); // For each particle, how useful have been
																		// its neighbours
	f_swarm=fopen("f_swarm.txt","w");

	E = exp ((long double) 1); 
	pi = acos ((long double) -1);
	
	//--------- Simulations
	SPSO=0; // 0 => use the parameters defined in paramBPSO.c
						// 1 => "simulate" Standard PSO 2007 (see paramSPSO07.c)
						// 2 => simulate SPSO-BK (Bratton, Kennedy). See paramSPSPBK.c
						
	param.explSave=0; // 1 => save the exploitation rate on f_expl
										// 0 => don't
										// See param.size_LEA, either in paramSPSO07 or paramBPSO
										// depending on your choice for the above SPSO07 value
	
	// ----------------------------------------------- PROBLEM
	functionCode = 102;
	/* (see problemDef, cellular_phone, cec2005pb for precise definitions)
	 0 Parabola (Sphere)
	 1 Griewank
	 2 Rosenbrock (Banana)
	 3 Rastrigin
	 4 Tripod (dimension 2)
	 5 Ackley
	 6 Center-bias test function
	 7 Pressure vessel (penalty method)
	 1007        "        (confinement method)
	 8 Compression spring
	 1008				"				 (confinement method)
	 9 Gear train
	 10 Cellular phone
	 11 PAPR/OFDM
	 12 Schwefel
	 13 Cutting stock
	 14 convex small polygon, 4 edges, smallest perimeter
	 15 constrained problem g13 (penalty method)
	 16 constrained problem g3 (penalty method)
	 1015 				"               (confinement method)

	 CEC 2005 benchmark  (no more than 30D. See cec2005data.c)
	 100 F1 (shifted Parabola/Sphere) 
	 102 F6 (shifted Rosenbrock) 
	 103 F9 (shifted Rastrigin) 
	 104 F2 Schwefel 
	 105 F7 Griewank  (NOT rotated)
	 106 F8 Ackley  (NOT rotated) 

	 99 Test

	 -1  Initialisation of N particles in a D-space => D*N variables
	 -2  idem, but with another potential function
	 */ 

	runMax = 100; // Numbers of runs


	// =========================================================== 

		pb=problemDef(functionCode,fLandscape);

	//printf("\n %f",paprExhaust(pb)); // Only for problem 11 and dimension 8
	//return(0);

	//#include "multimode_check.c" // Specific. Check multimodality for the
	// Celular phone problem
	
		  #include "paramBPSO.c"


	  if(param.normalise>0) // Normalised search space [0,1]^D
	{
		for (d = 0; d < pb.SS.D; d++) 
		{
			pb.SS.q.q[d] =pb.SS.q.q[d]/(pb.SS.max[d]-pb.SS.min[d]) ; // Normalise the granularity
			pb.SS.max[d]=1;
			pb.SS.min[d]=0;	
			pb.SS.maxS[d]=1;
			pb.SS.minS[d]=0;
		}
	}  

NisBD=pow(param.NisB,1./pb.SS.D); // Global variable 
																	// Just to avoid to compute it many times
																	
	// Display information about the problem and the parameters
		randOption=param.rand;	
		// Some information
		printf ("\n Function %i ", functionCode);
		printf("\n Dimension %i",pb.SS.D);
		if(param.swarmVar==1) printf("\n Initial s"); else printf("\n S");
		printf("warm size %i", param.S);
		printf("\n Mean size of the neighbourhood %f", param.p*param.S);

		printf("\n (clamping, R, randOrder, exploitation, stop_criterion) = (%i,%i, %i, %i, %i)",
		    param.clamping, param.R, param.randOrder,param.explOption,  param.stop);
		printf("\n randomness %i",param.rand);
		printf("\n init %i",param.init);
		printf("\n initVel %i",param.initVel);



		switch(param.W)
	{
		default:
		printf("\n w ");
		for(d=0;d<wSize;d++)
			printf(" %f",param.w[d]);
		
		 break;
		case 4:
			printf("\n w0Rate %f",param.w0Rate);	 
		case 5:
		printf("\n w in [%f, %f]",param.w[0],param.w[1]);	
			
			 break;
	}

		switch(param.C1)
		{
		default:
		if(param.W!=5)
			printf("\n c1 = %f, c2 = %f",param.c1,param.c2);
			else
			printf("\nc1 and c2 are variable");
			
		break;
		case 1:
		printf("\nc1 and c2 are variable");
		break;
		}
		
		if(param.R==1) printf("\nStandard deviation %f",param.sigma);

		printf("\nselect = %f",param.select);

		switch(param.choose_LEA)
	{
		default: 
		break;
		case 1: // LEA of some particles
		case 2: // best LEA
			printf("\n Forced LEA rate  %f",param.force_LEA); 
		printf("\n Exploitation area relative side length: %f",param.size_LEA);

		switch(param.draw_LEA)
		{
			default: 
			break;
			case 0: printf("\n middle of the LEA"); break;
			case 1: printf("\n uniform distribution"); break;
			case 2: printf("\n Gaussian distribution");break;
			case 3: printf("\n Either middle or uniform"); break;
	
			case 4: // Krigin
				param.aroundNb=66*pb.SS.D+33;
			if(param.aroundNb>LMax)
			{
				printf("\nWARNING. param.aroundNb should be %i, but has been reduced to %i",
				    param.aroundNb,LMax);
				printf("\n You should increase LMax");
				param.aroundNb=LMax;
			}

			printf("\n Local search by kriging. Variogram over at most %i points",
			    param.aroundNb);

			break;
		}
		break;
	}// End of choose_LEA


		switch(param.explOption)
	{
		default:
			break;
		case 3:
			printf("\n Exploitation area relative side length: either %f or %f",1./param.S,param.size_LEA);		
			break;
		case 4:
			printf("\n Exploitation area relative side length: uniform in [0,%f]",param.size_LEA);
			break;
		case 5:
			printf("\n Exploitation area relative side length: Gaussian (%f,%f)",1./param.S,param.size_LEA);
			break;
	}

		//---------------
		convRateMean=0;
		errorMean = 0;	    
		evalMean = 0;	    
		nFailure = 0;	
		//initLink=param.initLink;
		// Read the array for Orthogonal learning or Orthogonal Walk
		if(abs(param.strat[0])==2) 	orthoGenerate(pb.SS.D); 
		if(param.strat[0]==3 && param.OWmethod==2) 	orthoGenerate(pb.SS.D); 
		memo=0; 
		if(param.draw_LEA==4) memo=1;
		if(abs(param.strat[0])>2) memo=1;
		if(abs(param.strat[1])>2) memo=1;
		if(param.select>0) memo=1;
		if(param.swarmVar==1) memo=1;


		
		//------------------------------------- RUNS	
		for (run = 0; run < runMax; run++)  
	{	
		//printf("\n run %i/%i",run+1,runMax); 
				// For some options, positions must be memorised as much a possible
		fprintf(f_swarm,"run %i\n",run);
		FEclass=0;
		//if(initLink==2) param.initLink=alea_integer(0,1);
		//srand (clock () / 100);	// May improve pseudo-randomness 

		result = PSO (param, pb, 0);
		error = errorFC(result.error);

		convRateMean=convRateMean+result.convRate;

		if (error > pb.epsilon) // Failure
		{
			nFailure = nFailure + 1;
		}

		// Result display
		printf("\nmean %1.2e", errorMean/(run+1)); // Temporary error mean
		printf(", %0.0f %% ", (double)100*(run+1-nFailure)/(run+1)); // Temporary success rate
		printf (" %i/%i. Eval %f. Error %f ", run+1,runMax, result.nEval, error);
		printf("  x(0)= %f",(double)result.SW.P[result.SW.best].x[0]);

		// Save result
		fprintf( f_run, "\n%i %f %e ", run, result.nEval,  error );
		for ( d = 0; d < pb.SS.D; d++ ) fprintf( f_run, " %0.20f",  (double)result.SW.P[result.SW.best].x[d] );


		// Compute/store some statistical information
		if (run == 0)
		{
			errorMin = error;
			bestBest=result.SW.P[result.SW.best];
		}
		else if (error < errorMin)
		{
			errorMin = error;
			bestBest=result.SW.P[result.SW.best];
		}

		evalMean = evalMean + result.nEval;	
		errorMean = errorMean + error;	
		errorMeanBest[run] = error;
		logProgressMean  = logProgressMean - log(error);
	
	/*	
		// Save the "confidence" table for this run
		printf("\n Save the confidence table on f_confid.txt");
		for(s=0;s<param.S;s++)
		{
			for(s1=0;s1<param.S;s1++) 
			fprintf(f_confid,"%f ",(double)result.SW.confid[s].plus[s1]/result.SW.confid[s].minus[s1]);
			fprintf(f_confid,"* ");
			
			for(s1=0;s1<param.S;s1++) fprintf(f_confid,"%i ",result.SW.confid[s].plus[s1]);
			fprintf(f_confid,"* ");
			
			for(s1=0;s1<param.S;s1++) fprintf(f_confid,"%i ",result.SW.confid[s].minus[s1]);	
			fprintf(f_confid,"\n");
		}
		fprintf(f_confid,"\n");	
	*/	
	}		// End loop on "run"


		// Display statistical information
		// Means
		convRateMean=convRateMean/ (double) runMax; 
		evalMean = evalMean / (double) runMax;   
		errorMean = errorMean / (double) runMax;
		logProgressMean = logProgressMean/(double) runMax;

		printf ("\n Conv. rate (mean)= %f", convRateMean);		
		printf ("\n Eval. (mean)= %f", evalMean);	
		printf ("\n Error (mean) = %.12f, %.20e", errorMean,errorMean);

		// Variance
		variance = 0;

		for (run = 0; run < runMax; run++)
			variance = variance + pow (errorMeanBest[run] - errorMean, 2);

		variance = sqrt (variance / runMax);	    
		printf ("\n Std. dev. %.20f", variance); 
		printf("\n Log_progress (mean) = %f", logProgressMean);	

		// Success rate and minimum value
		printf("\n Failure(s) %i",nFailure);
		successRate = 1 - nFailure / (double) runMax;			
		printf ("\n Success rate = %.2f%%", 100*successRate);

		//if (run > 1)
		printf ("\n Best min value = %f", bestBest.f.f[0]);
		if(pb.constraint>0)
	{
		printf("%i constraints (should be negative): ",pb.constraint);
		for(n=0;n<pb.constraint;n++) printf("%f ",bestBest.f.f[n+1]);
	}

		if(param.normalise>0) // Back to the real search space
	{
		for ( d = 0; d < pb.SS.D; d++ )
			bestBest.x[d]=pb.SS.minR[d]+bestBest.x[d]*(pb.SS.maxR[d]-pb.SS.minR[d]);	
	}

		// Display the best position
		printf("\n Position: ");
		for ( d = 0; d < pb.SS.D; d++ ) printf( " %f", (double) bestBest.x[d] );

		// Save
		/*
		 fprintf(f_synth,"\n"); for (d=0;d<SS.D;d++) fprintf(f_synth,"%f ",
		     pb.offset[d]);
		 fprintf(f_synth,"    %f %f %f %.0f%% %f",errorMean,variance,errorMin,
		     successRate,evalMean); 

		 fprintf( f_synth, "\n%f %f %f %f %.0f%% %f ", shift,
		     errorMean, variance, errorMin, successRate, evalMean );
		 */	 	
		fprintf (f_synth, "%f %f %.0f%% %f   ",
		    errorMean, variance, 100*successRate, evalMean);		   

		// Save the best result
		fprintf(f_synth,"\n ");
		for(n=0;n<pb.constraint+1;n++) fprintf( f_synth, "%f  ", bestBest.f.f[n]);
		fprintf(f_synth," X");
		for ( d = 0; d < pb.SS.D; d++ ) fprintf( f_synth, " %f", (double) bestBest.x[d] );

		// Save the position in the "init" format, if functionCode<0
		if(functionCode<0)
	{
		dim=pb.SS.D/pb.nb;
		fprintf(f_init_save,"%i\n",pb.nb);
		for(n=0;n<pb.nb;n++)
		{
			for(d=0;d<dim;d++) 	fprintf(f_init_save,"%f ",(double) bestBest.x[d+n*dim]); 
			fprintf(f_init_save,"\n");
		}
	}

		// Save the curve "success rate" vs FEmax
		// Save the curve "mean best" vs FEmax

		for(j=0;j<FEmaxSize-1;j++)
	{
		zz=0;
		zz1=0;
		for(run=0;run<runMax;run++)
		{
			zz=zz+successFlag[run][j];
			zz1=zz1+bestS[run][j];
		}

		zz=zz/runMax; // success ratio
		zz1=zz1/runMax; // mean best

		fprintf(fCurve,"%i %e %f\n",FEmax[j],zz1,zz);
	}
		fprintf(fCurve, "%i %e %f", FEmax[FEmaxSize-1],errorMean, successRate);
		 
		

		//-----------------------------------------------------
		// Repeat some information about options, by using the codification
		// (by comparing to SPSO 07)
		printf("\n\n Function %i", functionCode); printf(" Dimension %i",pb.SS.D);
		if(param.swarmVar==1) printf("\nInitial s"); else printf("\nS");
		printf("warm size %i",param.S);
		printf("\nTopology %i",param.topology);
		if(param.topology==0)
			printf("\n param K for neighbourhood size: %i",param.K);
	//	if(param.initLink<3)
			printf(" Probability to be an informant = %0.2f",param.p);
		
		printf("\nResult obtained with PSO ");
		if(param.C1>0) printf(" C1(%i)",param.C1);
		if(param.C2>0) printf(" C2(%i)",param.C2);
			
		if(param.choose_LEA!=0) printf(" CH%i",param.choose_LEA);			
		if (param.clamping!=1) printf(" CL%i",param.clamping);
		if(param.draw_LEA!=0) printf(" D%i",param.draw_LEA);

		
		if(param.inheritance>0) printf(" INH%0.2f",param.inheritance);		
		if(param.topolLinkNbMax!=1 && param.swarmVar==0) printf(" TLM%i",param.topolLinkNbMax);
		//if(param.initLink>0) printf(" IL%i",param.initLink);
		if(param.topolSelf==0) printf(" TS0");
		
		if(param.mutation[0]+param.mutation[1]+param.mutation[2]>0)
			printf(" M(%0.2f, %0.2f, %0.2f)",param.mutation[0],param.mutation[1],
																				param.mutation[2]);
																				
		
		if(param.nonLinear>0)
	{ printf(" NL%i", param.nonLinear);
		printf("(%0.1f)",param.nonLinearCoeff[0]);
	}
		if(param.normalise!=0) printf(" NR%i",param.normalise);
		if(param.randOrder!=1) printf(" Ro%i",param.randOrder);		
		if(param.init !=0) printf(" P%i", param.init);
		
		if(param.quant>0) printf(" Q%i",param.quant);
		if(param.R!=0) 
		{
			printf(" R%i",param.R);
			switch(param.R)
			{
			case 1: printf("(%f)",param.sigma);break;
			case 2: case 3: printf("(%0.1f)",param.distPow);
			}
		}
	
		if(param.rand!=1) printf(" r%i",param.rand);
		if(param.select>0) printf(" SL(%f)",param.select);	
								
		if(param.initVel!=5) printf(" V%i", param.initVel);
		if(param.W>0) printf(" W%i",param.W);
		if(param.W==4) printf("(%0.2f)",param.w0Rate);
		if(param.opposite>0) printf(" Opp");
		
		if(param.strat[0]!=0 || param.strat[1]!=0) 
		{
			printf(" Strat(%i,%i)",param.strat[0],param.strat[1]);
			if(param.strat[0] != param.strat[1]) printf("(%f)",param.stratPop);
		}
		
		if(param.stagType!=0)
					printf(" stagType %i",param.stagType);
		if(param.stagType==3 ) printf("(%f)",param.stagRate);
		
		if(param.swarmVar==1) printf(" SwV");
		if(param.spreadProba>0) printf(" spreadProba %f",param.spreadProba);
		
		return 0; // End of main program
}



// =============================================================================
#include "alea.c"
#include "compare.c"
#include "confin.c"

#include "exploArea.c"
#include "init.c"

#include "modelFunctOptim.c"

#include "move.c"

#include "perf.c"

#include "problemDef.c"
#include "variogram.c"
#include "tools.c"

#include "swarmSelect.c"
#include "PSO.c"


//#include "papr.c" // Just for tests

