struct xv moveOW(int s, struct swarm SW, struct problem pb, struct param param,
									struct quantum qAdapt)
{
	// ---------------------------------------------------ORTHOGONAL WALK STRATEGY
	int d;
	// Build a new position Y by "orthogonal walk"

	double coeff1=1./(SW.S+1); //0.1; //0.5; 
	double coeff2=0.5/(SW.S-1);
	int D=SW.X[s].size;
	double dx[DMax];
	
	int improv;
	int j;

	double maxD,minD;

	
	int noExploit=1; // 0 => keep moving as long as there is improvement
										// 1 => stop after the first improvement
	int optionInit=2; // Re-init option
	// 1 => at random
	//* default => Try to find a "no-man's-land"
	int optionStep=1; // How to define the initial steps
	// 0 => according to the velocity
	//* 1 => according to the search space
	// 2 => according to the swarm
	int ss;
	double step[DMax][2];
	double stepMin[DMax]; 
	// For these two parameters, something adaptive or/and a theoretical analysis
	// would be welcome, for the given values are just rules of thumb
	double stepMinCoeff=0.5e-8; 
	double stepDecrease=0.4; 


	double stepIncrease=1./stepDecrease; 
	struct xv xv;
	struct position Y0,Y1, YBest;
	int zTot;


	// Define the step min for each dimension
	for(d=0;d<D;d++)
	{
		stepMin[d]=stepMinCoeff*(pb.SS.max[d]-pb.SS.min[d]);
	}

	YBest=SW.X[s];
	Y1.size=YBest.size;
	Y0=YBest;

	// Define the steps
	switch(optionStep)
	{
		case 0: // step = velocity
			for(d=0;d<D;d++)
		{
			step[d][0]=SW.V[s].v[d];
			step[d][1]=step[d][0];
		}
		break;

		case 1: // Depending on the search space
			for(d=0;d<D;d++)
		{
			step[d][0]=coeff1*(pb.SS.max[d]-pb.SS.min[d]);
			step[d][1]=step[d][0];
		}
		break;

		case 2: // Depending on the swarm

			for(d=0;d<D;d++)
		{
			// Find the min and the max in the swarm
			maxD=SW.X[0].x[d];
			minD=maxD;

			for(ss=1;ss<SW.S;ss++)
			{
				maxD=max(maxD,SW.X[ss].x[d]);
				minD=min(minD,SW.X[ss].x[d]);
			}
			step[d][0]=coeff2*(maxD-minD); 
			step[d][1]=step[d][0]; // For the moment, the simplest way
		}
		break;
	}

	improv=0;
	j=0;
	OW:  // ----------------------------------LOOP

		if(nEval >= param.effort*pb.evalMax) goto end; // If too many evluations, stop

	switch(param.OWmethod)
	{
		default: // At random, step along all dimensions, +dx or -dx
			for(d=0;d<D;d++) // Move
		{		
			if(alea(0,1)<0.5) dx[d]=step[d][0]; else dx[d]=-step[d][1];
			Y1.x[d]=Y0.x[d]+dx[d];
		}
		break;

		case 1: // At random, step along some dimensions
			for(d=0;d<D;d++)
		{		
			switch(alea_integer(0,2))
			{
				default: dx[d]=0; break;
				case 1: dx[d]=step[d][0]; break;
				case 2: dx[d]=-step[d][1]; break;
			}
			Y1.x[d]=Y0.x[d]+dx[d];
		}
		break;

		case 2: // Using the orthogonal array
			for(d=0;d<D;d++)
		{		
			if(orthoArray.a[j][d]==1) dx[d]=step[d][0];
			else dx[d]=-step[d][1];
			Y1.x[d]=Y0.x[d]+dx[d];
		}
		j=j+1; if(j>=orthoArray.size1) j=0;
		break;

	}
	// Evaluate
	Y1.f=perf (Y1, pb, 0, 0, 0,param.normalise);

	// Check improvement
	if(betterThan(Y1.f, YBest.f)==1)  // Improvement
	{ 		
		improv++;

		YBest=Y1;
		if(noExploit==1) goto end;
		if(errorFC(YBest.f)<pb.epsilon) goto end;			

		Y0=YBest; 

		// Increase the step
		for(d=0;d<D;d++) {step[d][0]=stepIncrease*step[d][0]; step[d][1]=stepIncrease*step[d][1];} 

		goto OW;
	}

	if(improv>=param.orthoWalkApply) goto end; // If "enough" improvements, stop

	// No improvement, decrease the step
	for(d=0;d<D;d++) {step[d][0]=stepDecrease*step[d][0]; step[d][1]=stepDecrease*step[d][1];} 

	// Check if not too small
	zTot=0;
	for(d=0;d<D;d++)
	{
		if(fabs(step[d][0])<stepMin[d] && fabs(step[d][1])<stepMin[d]) zTot++;
	}

	if(zTot<D) goto OW; // Not to small

	// Step too small => re-initialise

	switch(optionInit) // WARNING. X is modified
	{
		default:  SW.X[s]=initFar(pb, param, qAdapt); break; // (the archive is a global variable)
		case 1: SW.X[s]=initPos(pb,param,qAdapt); break; // At random
	}
	//printf("\nmoveOW 170 reinit");	


	end:
		// Compute the velocity, just for hybrid method
		// (for pure Orthogonal Walk, it is not necessary)
		for(d=0;d<pb.SS.D;d++) xv.v.v[d]=SW.X[s].x[d]-YBest.x[d];

		xv.x=YBest;
		return xv;
}	